import os
from box.sphinx import Settings, connect


class Settings(Settings):

    # Documentation:
    # http://sphinx-doc.org/config.html

    # General

    extensions = ['sphinx.ext.autodoc']
    master_doc = 'index'
    pygments_style = 'sphinx'

    # HTML

    @property
    def html_theme(self):
        if os.environ.get('READTHEDOCS', False):
            return 'default'
        return 'sphinx_rtd_theme'

    @property
    def html_theme_path(self):
        if os.environ.get('READTHEDOCS', False):
            return []
        import sphinx_rtd_theme
        return [sphinx_rtd_theme.get_html_theme_path()]

    # Autodoc

    autodoc_skip_members = []

    @connect('autodoc-process-docstring')
    def autodoc_process_docstring(self, app, what, name, obj, options, lines):
        if what != 'module':
            for key, value in enumerate(lines):
                if key >= 1 and value.startswith('---'):
                    lines[key - 1] = lines[key - 1].join(['**', '**'])
                    lines[key] = ''

    @connect('autodoc-skip-member')
    def autodoc_skip_member(self, app, what, name, obj, skip, options):
        return skip or (name in self.autodoc_skip_members)
