import os
import shutil
import inspect
import subprocess
from box.functools import Function, cachedproperty
from ..settings import settings


class create(Function):
    """Create packgram.

    Parameters
    ----------
    path: str
        Path where new packgram will be created.
    pattern: str
        Builtin pattern name or path to pattern.
    command: str
        Command to bootstrap packgram.
    """

    # Public

    default_command = settings.default_command
    default_pattern = settings.default_pattern

    def __init__(self, path, *, pattern=None, command=None):
        if pattern == None:
            pattern = self.default_pattern
        if command == None:
            command = self.default_command
        self._path = path
        self._pattern = pattern
        self._command = command

    def __call__(self):
        if os.path.exists(self._path):
            raise RuntimeError(
                'Path "{self._path}" already exists.'.format(self=self))
        try:
            shutil.copytree(self._pattern_path, self._path)
        except Exception:
            if os.path.exists(self._path):
                shutil.rmtree(self._path)
        else:
            subprocess.check_call(self._command,
                shell=True, cwd=self._path)

    # Protected

    @cachedproperty
    def _pattern_path(self):
        if os.path.sep not in self._pattern:
            # Trying to use builtin pattern
            path = os.path.join(self._patterns_path, self._pattern)
            if os.path.isdir(path):
                return path
        return self._pattern

    @property
    def _patterns_path(self):
        selfdir = os.path.dirname(inspect.getfile(type(self)))
        return os.path.join(selfdir, '..', 'patterns')
